{% extends "base.html" %}

{% block title %}
	&raquo; CSV Sites HH Data
{% endblock %}

{% block nav %}
	CSV Sites HH Data
{% endblock %}

{% block content %}
	<form action="/reports/183" method='post'>
		<fieldset>
			<legend>Download</legend>	
			<label>Type</label>
			<select name="type">
				<option value="used">Used</option>
				<option value="imp_grid">Imported</option>
				<option value="exp_grid">Exported</option>
				<option value="imp_gen">Generated</option>
				<option value="displaced">Displaced</option>
				<option value="exp_gen">Parasitic</option>
			</select>
			<label>From</label> {{input_date('start', start_date, resolution='day')}}
			<label>To</label> {{input_date('finish', finish_date, resolution='day')}}
			<label>Site Codes</label>
			{{input_textarea('site_codes', '', 5, 40,
			  placeholder='One on each line, includes all if left blank')}}    						
			<input type="submit" value="Download">
		</fieldset>
	</form>

{% endblock %}
