{% extends "base.html" %}

{% block title %}
	&raquo; CSV Supplies Snapshot
{% endblock %}

{% block nav %}
	CSV Supplies Snapshot
{% endblock %}

{% block content %}

	<form action="/reports/33">
		<fieldset>
			<legend>Download supply info for a point in time</legend>	
			<label>At</label> {{input_date('date', default_timestamp)}}
			<label>MPAN Cores</label>
			{{input_textarea(
				'mpan_cores', '', 5, 40,
				placeholder='One on each line, includes all supplies if left blank')}}
			<input type="submit" value="Download">
		</fieldset>
	</form>

	<p>
		It's mandatory that a supply is HH if the import or export average of the three
	 	highest monthly maximum demands over the past 12 complete months is &gt; 100 kW.
	</p>

{% endblock %}
