	{% extends "base.html" %}

{% block title %}
	&raquo; Supplies &raquo; {{supply.id}} &raquo; Notes &raquo; Add
{% endblock %}

{% block nav %}
	<a href="/e/supplies">Supplies</a>  &raquo;
	<a href="/e/supplies/{{supply.id}}">{{supply.id}}</a>  &raquo;
	<a href="/e/supplies/{{supply.id}}/notes">Notes</a>  &raquo; Add
{% endblock %}

{% block content %}
	<form method="post">
		<fieldset>
			<legend>Note</legend>
			<label>Body</label> {{input_textarea('body', '', 30, 80)}}
			<input type="submit" value="Add">
		</fieldset>
	</form>	
{% endblock %}
