{% extends "base.html" %}

{% block title %}
	&raquo; System &raquo; Object Summary
{% endblock %}

{% block nav %}
	<a href="/system">System</a> &raquo; Object Summary
{% endblock %}

{% block content %}
	<pre>{{summary}}</pre>
{% endblock %}
