{% extends "base.html" %}

{% block title %}
	&raquo; Reading Frequencies &raquo; {{g_reading_frequency.code}}
{% endblock %}

{% block nav %}
	<a href="/g_reading_frequencies">Reading Frequencies</a> &raquo;
 	{{g_reading_frequency.code}}
{% endblock %}

{% block content %}

	<table>
		<tr>
			<th>Code</th>
			<td>{{g_reading_frequency.code}}</td>
		</tr>
		<tr>
			<th>Description</th>
			<td>{{g_reading_frequency.description}}</td>
		</tr>
	</table>

{% endblock %}
