{% extends "base.html" %}

{% block title %}&raquo; Sites &raquo; Add{% endblock %}

{% block nav %}
	<a href="/sites">Sites</a> &raquo; Add
{% endblock %}

{% block content %}
	<form method="post">
		<fieldset>
			<legend>Add a site</legend>
			<label>Code</label> {{ input_text('code', '') }}
			<label>Name</label> {{ input_text('name', '') }}
			<input type="submit" value="Add">
		</fieldset>
	</form>
{% endblock %}
