{% extends "base.html" %}

{% block title %}
	&raquo; MTCs
{% endblock %}

{% block nav %}
	MTCs
{% endblock %}

{% block content %}
	<table>
		<caption>Meter Timeswitch Classes</caption>	
		<thead>
			<tr>
				<th>View</th>
				<th>Code</th>
				<th>Common?</th>
				<th>Related Metering?</th>
				<th>Valid From</th>
				<th>Valid To</th>
			</tr>
		</thead>
		<tbody>
			{% for mtc in mtcs %}
				<tr>
					<td><a href="/e/mtcs/{{mtc.id}}">view</a></td>
					<td>{{mtc.code}}</td>
					<td>
						{% if mtc.is_common %}
							Yes
						{% else %}
							No
						{% endif %}
					</td>
					<td>
						{% if mtc.has_related_metering %}
							Yes
						{% else %}
							No
						{% endif %}
					</td>
					<td>{{mtc.valid_from|hh_format}}</td>
					<td>{{mtc.valid_to|hh_format}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
